'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _helper = require('./data/redux/helper');

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/**
 * Used to register components to use for component browser.
 *
 * @SERVER
 */
var ComponentBrowserRegister = function () {

  /**
   * @param {Logger} logger
   */

  /**
   * @type {Logger}
   */
  function ComponentBrowserRegister(logger) {
    _classCallCheck(this, ComponentBrowserRegister);

    this._logger = logger;
    this._components = {};
  }

  /**
   * Example:
   *
   * {
   *   vendor: {
   *     ComponentName: {
   *       thumbnail: '/.../.../../ComponentNameThumb.js',
   *       test: '/../../../ComponentNameThumb.js',
   *       detail: {
   *         // Component JSON.
   *       }
   *     }
   *   }
   * }
   *
   * @type {{[key: string]: {[key:string]: ComponentJson}}}
   */


  _createClass(ComponentBrowserRegister, [{
    key: 'reg',
    value: function reg(vendorName, componentName, data) {
      if (!this._components.hasOwnProperty(vendorName)) {
        this._components[vendorName] = {};
      }
      if (this._components[vendorName].hasOwnProperty(componentName)) {
        this._logger.notice('Duplicate component, will be overwritten: ' + vendorName + '.' + componentName);
      }
      this._components[vendorName][componentName] = data;
    }

    /**
     * @param {string} absoluteRootDir
     * @param {Object} componentJson
     */

  }, {
    key: 'register',
    value: function register(absoluteRootDir, componentJson) {
      var isValid = this._validateComponentJson(componentJson);
      if (!isValid) {
        this._logger.notice("Invalid component json file: " + _path2.default.join(absoluteRootDir, 'component.json'));
        return;
      }

      if (!this._components.hasOwnProperty(componentJson.vendor)) {
        this._components[componentJson.vendor] = {};
      }
      if (!this._components[componentJson.vendor].hasOwnProperty(componentJson.name)) {
        this._logger.notice('Duplicate component, will be overwritten: ' + componentJson.vendor + '.' + componentJson.name);
      }

      var thumbnailPath = _path2.default.join(absoluteRootDir, componentJson.thumbnail);
      var testPath = null;
      if (componentJson.test != null && (0, _helper.isStringDuckType)(componentJson.test) && componentJson.test != '') {
        testPath = _path2.default.join(absoluteRootDir, componentJson.test);
      }

      this._components[componentJson.vendor][componentJson.name] = {
        thumbnail: thumbnailPath,
        test: testPath,
        detail: componentJson
      };
    }
  }, {
    key: 'getComponents',
    value: function getComponents() {
      return this._components;
    }

    /**
     * @param {Object} componentJson
     * @return {boolean}
     */

  }, {
    key: '_validateComponentJson',
    value: function _validateComponentJson(componentJson) {
      var vendor = componentJson.vendor;
      var name = componentJson.name;
      var thumbnail = componentJson.thumbnail;

      return vendor != null && (0, _helper.isStringDuckType)(vendor) && vendor != '' && name != null && (0, _helper.isStringDuckType)(name) && name != '' && thumbnail != null && (0, _helper.isStringDuckType)(thumbnail) && thumbnail != '';
    }
  }]);

  return ComponentBrowserRegister;
}();

exports.default = ComponentBrowserRegister;
//# sourceMappingURL=ComponentBrowserRegister.js.map