'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/**
 * @type {RegExp}
 */
var UNDERSCORE_REGEX = /^_/;

/**
 * @SERVER
 */

var PageFinder = function () {

  /**
   * @param {Logger} logger
   */
  function PageFinder(logger) {
    _classCallCheck(this, PageFinder);

    this._logger = logger;
  }

  /**
   * 1) Find recursively every component in the directory.
   * 2) Validate the component.
   *
   * @param {string} absoluteRootDir
   * @param {Function} callback
   */


  _createClass(PageFinder, [{
    key: 'find',
    value: function find(absoluteRootDir, callback) {
      var i, vendorNames;
      this._check(absoluteRootDir, true, false);
      vendorNames = _fs2.default.readdirSync(absoluteRootDir);
      for (i = 0; i < vendorNames.length; i++) {
        this._checkComponentDir(absoluteRootDir, vendorNames[i], callback);
      }
    }
  }, {
    key: '_checkComponentDir',
    value: function _checkComponentDir(absoluteRootDir, vendorName, callback) {
      var i, filenames, vendorRootDir, compName, compRootDir, compMain;
      vendorRootDir = _path2.default.join(absoluteRootDir, vendorName);
      this._check(vendorRootDir, true, false);
      filenames = _fs2.default.readdirSync(vendorRootDir);
      for (i = 0; i < filenames.length; i++) {
        compName = filenames[i];
        compRootDir = _path2.default.join(vendorRootDir, compName);
        compMain = _path2.default.join(vendorRootDir, compName, compName + '.js');

        if (!this._check(compRootDir, true, true)) continue;
        if (!this._check(compMain, false, true)) continue;

        if (UNDERSCORE_REGEX.test(compName)) {
          throw new Error('Component name starting with underscore is not allowed: ' + compName);
        }

        callback(vendorName, compName, compRootDir, _path2.default.join(vendorName, compName));
      }
    }

    /**
     * @param {string} absPath
     * @param {boolean} shouldBeDir
     * @param {boolean} shouldIgnore
     * @return {boolean}
     */

  }, {
    key: '_check',
    value: function _check(absPath, shouldBeDir, shouldIgnore) {
      var stat;
      try {
        stat = _fs2.default.statSync(absPath);
      } catch (error) {
        if (shouldIgnore) {
          this._logger.notice('Ignored: Component file/dir does no exist: \'' + absPath + '\'.');
          return false;
        } else {
          throw error;
        }
      }

      var isMismatchType = shouldBeDir && !stat.isDirectory() || !shouldBeDir && !stat.isFile();
      if (isMismatchType) {
        if (shouldIgnore) {
          this._logger.notice(this._mismatchTypeMessage(absPath, shouldBeDir, 'Ignored: '));
          return false;
        } else {
          throw new Error(this._mismatchTypeMessage(absPath, shouldBeDir, ''));
        }
      }
      return true;
    }

    /**
     * @param {string} absPath
     * @param {string} shouldBeDir
     * @param {string} prefix
     */

  }, {
    key: '_mismatchTypeMessage',
    value: function _mismatchTypeMessage(absPath, shouldBeDir, prefix) {
      if (shouldBeDir) {
        return prefix + 'Expected directory, but file found: \'' + absPath + '\'.';
      }
      return prefix + 'Expected file, but directory found: \'' + absPath + '\'.';
    }
  }]);

  return PageFinder;
}();

exports.default = PageFinder;
//# sourceMappingURL=PageFinder.js.map