'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

// The reason we use full path is to make webpack's resolve.alias work.
// We need to replace custom node registration with user file so that
// custom nodes can be loaded in both client and server side.


var _ReverseRouter = require('../router/ReverseRouter.js');

var _ReverseRouter2 = _interopRequireDefault(_ReverseRouter);

var _NodeFactory = require('../router/NodeFactory.js');

var _NodeFactory2 = _interopRequireDefault(_NodeFactory);

var _DomainNode = require('../router/DomainNode.js');

var _DomainNode2 = _interopRequireDefault(_DomainNode);

var _MethodNode = require('../router/MethodNode.js');

var _MethodNode2 = _interopRequireDefault(_MethodNode);

var _PathNode = require('../router/PathNode.js');

var _PathNode2 = _interopRequireDefault(_PathNode);

var _ClientHttpRequest = require('../http/ClientHttpRequest.js');

var _ClientHttpRequest2 = _interopRequireDefault(_ClientHttpRequest);

var _ClientCookieJar = require('../http/ClientCookieJar.js');

var _ClientCookieJar2 = _interopRequireDefault(_ClientCookieJar);

var _Provider = require('../Provider.js');

var _Provider2 = _interopRequireDefault(_Provider);

var _registerRouterNodes = require('soya/lib/server/registerRouterNodes');

var _registerRouterNodes2 = _interopRequireDefault(_registerRouterNodes);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/**
 * Responsible for client runtime.
 *
 * @CLIENT
 */
var SoyaClient = function () {

  /**
   * @param {Object} clientConfig
   */


  /**
   * @type {{[key: string]: Store}}
   */


  /**
   * @type {ReverseRouter}
   */


  /**
   * @type {Function}
   */


  /**
   * @type {void | string}
   */
  function SoyaClient(clientConfig) {
    _classCallCheck(this, SoyaClient);

    this._clientConfig = clientConfig;
    this._cookieJar = new _ClientCookieJar2.default();
    var nodeFactory = new _NodeFactory2.default();
    nodeFactory.registerNodeType(_DomainNode2.default);
    nodeFactory.registerNodeType(_MethodNode2.default);
    nodeFactory.registerNodeType(_PathNode2.default);
    // TODO: Make sure swapping for custom nodes also work at client side.
    (0, _registerRouterNodes2.default)(nodeFactory, clientConfig);

    this._reverseRouter = new _ReverseRouter2.default(nodeFactory);
    this._provider = new _Provider2.default(clientConfig, this._reverseRouter, false);
    this._pages = {};
    this._storeCache = {};
  }

  /**
   * @param {{[key: string]: Array<Array<any>>}} routeConfigMap
   */


  /**
   * @type {ClientCookieJar}
   */


  /**
   * @type {Provider}
   */


  /**
   * @type {Object}
   */


  /**
   * @type {void | {httpRequest: ClientHttpRequest; routeArgs: Object; hydratedState: Object}}
   */

  /**
   * @type {{[key: string]: Page}}
   */


  _createClass(SoyaClient, [{
    key: 'addRouteConfig',
    value: function addRouteConfig(routeConfigMap) {
      if (!routeConfigMap) return;
      var routeId;
      for (routeId in routeConfigMap) {
        if (!routeConfigMap.hasOwnProperty(routeId)) continue;
        this._reverseRouter.reg(routeId, routeConfigMap[routeId]);
      }
    }

    /**
     * @param {Function} pageClass
     */

  }, {
    key: 'register',
    value: function register(pageClass) {
      // We do overrides just in case there are updates.
      // TODO: Figure out history API navigation.
      this._pages[pageClass.pageName] = pageClass;

      if (this._currentPageName == pageClass.pageName) {
        // If a registration is made for currently active page, it means we're
        // hot-reloading. TODO: Reconcile this with history API navigation later.
        this.refresh();
      }
    }

    /**
     * Re-run rendering for the current page.
     */

  }, {
    key: 'refresh',
    value: function refresh() {
      var pageClass = this._getPageClass(this._currentPageName);
      var pageArgs = this._currentPageArgs;
      if (!pageArgs) {
        throw new Error('Unable to refresh, current page arguments does not exist!');
      }
      this._dismantleCurrentPage();
      this._render(pageClass, pageArgs);
    }

    /**
     * @param {string} name
     * @param {Object} routeArgs
     * @param {?any} hydratedState
     */

  }, {
    key: 'navigate',
    value: function navigate(name, routeArgs, hydratedState) {
      this._currentPageName = name;
      this._currentPageArgs = null;

      var pageClass = this._getPageClass(name);
      var httpRequest = new _ClientHttpRequest2.default();
      var pageArgs = {
        httpRequest: httpRequest,
        routeArgs: routeArgs,
        hydratedState: hydratedState
      };
      this._currentPageArgs = pageArgs;
      this._dismantleCurrentPage();
      this._render(pageClass, this._currentPageArgs);
    }

    /**
     * @param {Function} pageClass
     * @param {Object} pageArgs
     */

  }, {
    key: '_render',
    value: function _render(pageClass, pageArgs) {
      // Create new page instance - dependencies should be fetched and cached
      // by Provider.
      var page = new pageClass(this._provider, this._cookieJar, false);
      var storeNamespace = this._getPageStoreNamespace(pageClass);

      var store = this._storeCache[storeNamespace];
      if (!store) {
        store = page.createStore(pageArgs.hydratedState);
      }
      var hasStore = !!store;

      if (hasStore) {
        this._storeCache[storeNamespace] = store;
        if (module.hot) {
          store._mayHotReloadSegments();
        }
      }

      page.render(pageArgs.httpRequest, pageArgs.routeArgs, store, this._renderCallback.bind(this, store));
    }

    /**
     * Extract state from the currently active store.
     *
     * @return {any}
     */

  }, {
    key: '_extractState',
    value: function _extractState() {
      var store = this._getCurrentStore();
      return store._getState();
    }

    /**
     * @param {any} newState
     */

  }, {
    key: '_replaceState',
    value: function _replaceState(newState) {
      var store = this._getCurrentStore();
      store._replaceState(newState);
    }

    /**
     * @param {void | Store} store
     * @param {RenderResult} renderResult
     */

  }, {
    key: '_renderCallback',
    value: function _renderCallback(store, renderResult) {
      // Start rendering, this also do Segment registrations.
      window.renderResult = renderResult;
      if (store) store._startRender();
      this._currentPageDismantle = renderResult.contentRenderer.render();
      if (store) store._endRender();
    }

    /**
     * @param {string} name
     */

  }, {
    key: '_getPageClass',
    value: function _getPageClass(name) {
      var pageClass = this._pages[name];
      if (!pageClass) {
        throw new Error('Call to non-existent or non-loaded page: ' + name);
      }
      return pageClass;
    }

    /**
     * @param {Page} pageClass
     * @return {string}
     */

  }, {
    key: '_getPageStoreNamespace',
    value: function _getPageStoreNamespace(pageClass) {
      var storeNamespace = '__default';
      if (typeof pageClass.getStoreNamespace == 'function') {
        storeNamespace = pageClass.getStoreNamespace();
      }
      return storeNamespace;
    }

    /**
     * @returns {Store}
     */

  }, {
    key: '_getCurrentStore',
    value: function _getCurrentStore() {
      var pageClass = this._getPageClass(this._currentPageName);
      var storeNamespace = this._getPageStoreNamespace(pageClass);
      return this._storeCache[storeNamespace];
    }
  }, {
    key: '_dismantleCurrentPage',
    value: function _dismantleCurrentPage() {
      if (this._currentPageDismantle) this._currentPageDismantle();
    }
  }]);

  return SoyaClient;
}();

exports.default = SoyaClient;
//# sourceMappingURL=SoyaClient.js.map