'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _Compiler2 = require('../Compiler');

var _Compiler3 = _interopRequireDefault(_Compiler2);

var _CompileResult = require('../CompileResult');

var _CompileResult2 = _interopRequireDefault(_CompileResult);

var _EntryPoint = require('../../EntryPoint');

var _EntryPoint2 = _interopRequireDefault(_EntryPoint);

var _WebpackAssetServer = require('./WebpackAssetServer');

var _WebpackAssetServer2 = _interopRequireDefault(_WebpackAssetServer);

var _defaultFrameworkConfig = require('../../defaultFrameworkConfig.js');

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _rimraf = require('rimraf');

var _rimraf2 = _interopRequireDefault(_rimraf);

var _extractTextWebpackPlugin = require('extract-text-webpack-plugin');

var _extractTextWebpackPlugin2 = _interopRequireDefault(_extractTextWebpackPlugin);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var COMMONS_ENTRY_NAME = '__commons';

var SOURCE_MAP_FILE_REGEX = /\.[a-zA-Z0-9]{1,4}\.map/;
var CSS_FILE_REGEX = /\.css[\?#]?/;
var JS_FILE_REGEX = /\.js[\?#]?/;

/**
 * @SERVER
 */

var WebpackCompiler = function (_Compiler) {
  _inherits(WebpackCompiler, _Compiler);

  /**
   * @param {Logger} logger
   * @param {Object} frameworkConfig
   * @param {any} webpack
   * @param {any} react
   * @param {Function} webpackDevMiddleware
   * @param {Function} webpackHotMiddleware
   */


  /**
   * @type {?Function}
   */


  /**
   * @type {Logger}
   */


  /**
   * @type {any}
   */


  /**
   * @type {Array<Function>}
   */


  /**
   * @type {string}
   */

  /**
   * @type {{[key: string]: any}}
   */
  function WebpackCompiler(logger, frameworkConfig, webpack, react, webpackDevMiddleware, webpackHotMiddleware) {
    _classCallCheck(this, WebpackCompiler);

    var _this = _possibleConstructorReturn(this, (WebpackCompiler.__proto__ || Object.getPrototypeOf(WebpackCompiler)).call(this));

    _this._logger = logger;
    _this._react = react;
    _this._webpack = webpack;
    _this._webpackDevMiddleware = webpackDevMiddleware;
    _this._webpackHotMiddleware = webpackHotMiddleware;
    _this._frameworkConfig = frameworkConfig;
    _this._clientReplace = frameworkConfig.clientReplace;
    _this._clientResolve = frameworkConfig.clientResolve;
    _this._absoluteClientBuildDir = _path2.default.join(frameworkConfig.absoluteProjectDir, 'build/client');
    _this._assetHostPath = frameworkConfig.assetHostPath;
    _this._assetServer = new _WebpackAssetServer2.default(_this._assetHostPath, _this._absoluteClientBuildDir, logger);
    _this._firstCompileDone = false;

    if (_this._frameworkConfig.hotReload && (!webpackDevMiddleware || !webpackHotMiddleware)) {
      throw new Error('Hot reload flag is true, yet webpack dev/hot middleware is not passed to WebpackCompiler.');
    }

    // Only clean client build dir when precompileClient is not true
    if (!_this._frameworkConfig.precompileClient) {
      _this._cleanTempDir();
    }
    return _this;
  }

  /**
   * @param {any} webpack
   * @param {Object} frameworkConfig
   * @returns {Object}
   */


  /**
   * @type {boolean}
   */


  /**
   * @type {?Function}
   */


  /**
   * @type {any}
   */


  /**
   * @type {WebpackAssetServer}
   */


  /**
   * @type {{[key: string]: string}}
   */


  /**
   * @type {string}
   */


  _createClass(WebpackCompiler, [{
    key: 'getBabelLoaderExtras',


    /**
     * Only called at client side. Adds babel plugins like react-transform and
     * client resolve operations.
     *
     * @param {Object<string, boolean>} entryPointAbsolutePathMap
     * @return {Object}
     */
    value: function getBabelLoaderExtras(entryPointAbsolutePathMap) {
      var result = {
        plugins: []
      };

      if (this._frameworkConfig.hotReload) {
        result.plugins.push(['soya/lib/compiler/webpack/hotModuleAcceptPlugin.js', {
          hotReload: this._frameworkConfig.hotReload,
          entryPointAbsolutePathMap: entryPointAbsolutePathMap
        }]);
      }

      // Plugin no longer working, since Babel 6 runs plugins before presets.
      // result.plugins.push(['./hotModuleAcceptPlugin.js', {
      //  clientResolve: this._clientResolve
      // }]);

      if (this._frameworkConfig.hotReload) {
        // result.plugins.push('react-transform');
      }
      return result;
    }
  }, {
    key: 'run',


    /**
     * 1) Create configuration with one common file for webpack.
     * 2) Compile webpack from source to temporary location, with CSS loader
     *    and file loader.
     * 3) Read all files to AssetServer - AssetServer could just read the
     *    files, but cache everything.
     *
     *    TODO: Make babel optional
     *    TODO: Allow user setting of loaders.
     *
     * @param {Array<EntryPoint>} entryPoints
     * @param {Function} updateCompileResultCallback
     * @return {Array<Function>}
     */
    value: function run(entryPoints, updateCompileResultCallback) {
      var _this2 = this;

      var shouldCompile = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : true;

      var i,
          j,
          entryPoint,
          entryPointList = [];
      var configuration = {
        entry: {},
        output: {
          path: this._absoluteClientBuildDir,
          filename: "[name]-[chunkhash].js",
          publicPath: this._frameworkConfig.assetProtocol + '://' + this._assetHostPath
        },
        module: {
          loaders: [WebpackCompiler.getBabelLoaderConfig(), WebpackCompiler.getFileLoaderConfig(this._frameworkConfig)],
          noParse: /node_modules\/quill\/dist/
        },
        devtool: "source-map",
        resolve: WebpackCompiler.generateResolveConfig(this._frameworkConfig),
        plugins: [new this._webpack.optimize.OccurenceOrderPlugin()]
      };

      var cssModuleArgs = JSON.stringify({
        sourceMap: true,
        modules: true,
        localIdentName: this._frameworkConfig.debug ? '[name]_[local]_[hash:base64:3]' : '[hash:base64]'
      });

      // Links you need to read to understand this CSS section:
      // - https://webpack.github.io/docs/stylesheets.html
      // - https://github.com/webpack/extract-text-webpack-plugin
      // - https://webpack.github.io/docs/list-of-plugins.html
      // - https://github.com/webpack/css-loader
      // - https://github.com/webpack/style-loader
      var modulesCssLoader = {
        test: /\.mod\.css$/,
        loader: 'style-loader!css-loader?' + cssModuleArgs
      };
      var normalCssLoader = {
        test: /\.css$/,
        loader: 'style-loader!css-loader',
        exclude: /\.mod\.css$/
      };
      var modulesScssLoader = {
        test: /\.mod\.scss$/,
        loader: 'style-loader!css-loader?' + cssModuleArgs + '!sass-loader'
      };
      var normalSassLoader = {
        test: /\.scss$/,
        loader: 'style-loader!css-loader!sass-loader',
        exclude: /\.mod\.scss$/
      };
      if (!this._frameworkConfig.hotReload) {
        // Enable loading CSS as files.
        modulesCssLoader.loader = _extractTextWebpackPlugin2.default.extract("style-loader", "css-loader?" + cssModuleArgs);
        normalCssLoader.loader = _extractTextWebpackPlugin2.default.extract("style-loader", "css-loader");
        modulesScssLoader.loader = _extractTextWebpackPlugin2.default.extract("style-loader", "css-loader?" + cssModuleArgs + "!sass-loader");
        normalSassLoader.loader = _extractTextWebpackPlugin2.default.extract("style-loader", "css-loader!sass-loader");
        configuration.plugins.push(new _extractTextWebpackPlugin2.default('css/[name]-[chunkhash].css'));
      }
      configuration.module.loaders.push(modulesCssLoader);
      configuration.module.loaders.push(normalCssLoader);
      configuration.module.loaders.push(modulesScssLoader);
      configuration.module.loaders.push(normalSassLoader);

      for (i in this._clientReplace) {
        if (!this._clientReplace.hasOwnProperty(i)) continue;
        configuration.resolve.alias[i] = this._clientReplace[i];
      }

      if (this._frameworkConfig.hotReload) {
        configuration.plugins.push(new this._webpack.HotModuleReplacementPlugin());
      }

      configuration.plugins.push(new this._webpack.NoErrorsPlugin());
      configuration.plugins.push(new this._webpack.optimize.CommonsChunkPlugin({
        name: COMMONS_ENTRY_NAME,
        filename: 'common-[hash].js',
        minChunks: this._frameworkConfig.commonFileThreshold
      }));

      if (this._frameworkConfig.minifyJs) {
        configuration.plugins.push(new this._webpack.optimize.UglifyJsPlugin({}));
      }

      // Accept env var
      var definePlugin = new this._webpack.DefinePlugin({
        'process.env.NODE_ENV': this._frameworkConfig.NODE_ENV || '"development"'
      });
      configuration.plugins.push(definePlugin);

      var pageToRequire,
          entryPointAbsolutePathMap = {};
      for (i = 0; i < entryPoints.length; i++) {
        entryPoint = entryPoints[i];
        pageToRequire = _path2.default.join(entryPoint.absolutePath);
        entryPointAbsolutePathMap[pageToRequire] = true;
        configuration.entry[entryPoint.name] = pageToRequire;
        entryPointList.push(entryPoint.name);
      }

      // Add babel plugins.
      configuration.babel = this.getBabelLoaderExtras(entryPointAbsolutePathMap);

      var compiler = this._webpack(configuration);
      var self = this;

      compiler.plugin('done', function (stats) {
        if (stats.compilation.errors.length > 0) {
          // Error occured. Print error messages.
          _this2._printErrorMessages(stats);
          return;
        }

        var key;
        for (key in entryPointAbsolutePathMap) {
          if (!entryPointAbsolutePathMap.hasOwnProperty(key)) continue;
          entryPointAbsolutePathMap[key] = true;
        }

        var i,
            chunkMap = {};
        for (i = 0; i < stats.compilation.chunks.length; i++) {
          chunkMap[stats.compilation.chunks[i].name] = stats.compilation.chunks[i];
        }
        var commonsChunk = chunkMap[COMMONS_ENTRY_NAME];

        var compileResult = new _CompileResult2.default(),
            entryPointChunk,
            entryPointName,
            pageDep,
            depArray;
        for (i = 0; i < entryPointList.length; i++) {
          entryPointName = entryPointList[i];
          if (!chunkMap.hasOwnProperty(entryPointName)) {
            var error = new Error('Unable to determine dependency, entry point ' + entryPointName + ' does not have webpack chunk!');
            if (_this2._firstCompileDone) {
              // If this is a webpack hot reload compilation, don't throw the
              // error, just log to the user.
              _this2._logger.error('Hot reload ->', error);
              return;
            }
            throw error;
          }
          entryPointChunk = chunkMap[entryPointName];

          pageDep = {
            cssDependencies: [],
            jsDependencies: []
          };

          for (j = 0; j < commonsChunk.files.length; j++) {
            if (SOURCE_MAP_FILE_REGEX.test(commonsChunk.files[j])) continue;
            depArray = CSS_FILE_REGEX.test(commonsChunk.files[j]) ? pageDep.cssDependencies : pageDep.jsDependencies;
            depArray.push(self._assetServer.toUrlWithProtocol(commonsChunk.files[j], _this2._frameworkConfig.assetProtocol));
          }
          for (j = 0; j < entryPointChunk.files.length; j++) {
            if (SOURCE_MAP_FILE_REGEX.test(entryPointChunk.files[j])) continue;
            depArray = CSS_FILE_REGEX.test(entryPointChunk.files[j]) ? pageDep.cssDependencies : pageDep.jsDependencies;
            depArray.push(self._assetServer.toUrlWithProtocol(entryPointChunk.files[j], _this2._frameworkConfig.assetProtocol));
          }

          compileResult.pages[entryPointName] = pageDep;
        }

        // Sets the flag, we have completed the very first compilation. Next
        // compilation error should not trigger errors.
        _this2._firstCompileDone = true;

        updateCompileResultCallback(compileResult);
      });

      compiler.plugin('failed', function (err) {
        // Error ocurred. Print error messages.
        _this2._printErrorMessages(err);
      });

      var middlewares = [];

      if (this._frameworkConfig.hotReload) {
        // Let webpack-dev-middleware handles watching, storing output and serving
        // asset requests. Meanwhile, webpack hot middleware should be accepting
        // connection from the client.
        middlewares.push(this._webpackDevMiddleware(compiler, {
          noInfo: true,
          publicPath: configuration.output.publicPath
        }));
        middlewares.push(this._webpackHotMiddleware(compiler));
      } else {
        // Middleware for regular webpack asset server.
        middlewares.push(function (request, response, next) {
          var handledByAssetServer = _this2._assetServer.handle(request, response);
          if (!handledByAssetServer) {
            next();
          }
        });

        // Shouldn't be called on start when client code already precompiled
        if (shouldCompile) {
          // We only need to run if we are not hot reloading, since
          // webpack-dev-middleware already runs watch() for us.
          compiler.run(function () {
            // No-op. We'll use the above 'done' and 'failed' hook to notify
            // application for successes and failures.
          });
        }
      }

      return middlewares;
    }
  }, {
    key: '_cleanTempDir',
    value: function _cleanTempDir() {
      // Remove everything in the directory.
      _rimraf2.default.sync(this._absoluteClientBuildDir);

      // Ensure directory exists.
      try {
        _fs2.default.mkdirSync(this._absoluteClientBuildDir);
      } catch (e) {
        if (e.code != 'EEXIST') throw e;
      }
    }

    /**
     * @param {any} stats
     */

  }, {
    key: '_printErrorMessages',
    value: function _printErrorMessages(stats) {
      if (this._firstCompileDone) {
        // Second compilation is only done when we are using hot reload.
        // We should not stop the server if an exception happens.
        return;
      }
      var error = new Error('Webpack compilation error!');
      this._logger.error('Webpack compilation error -> ', error, [stats.compilation.errors]);
      throw error;
    }
  }, {
    key: '_createResolvePlugin',
    value: function _createResolvePlugin() {
      var self = this;
      return function (_ref) {
        var Plugin = _ref.Plugin,
            t = _ref.types;

        return new Plugin("soya-resolve-plugin", {
          visitor: {}
        });
      };
    }

    /**
     * @param {Object} frameworkConfig
     * @param {Object<string, boolean>} entryPointAbsolutePathMap
     */

  }, {
    key: '_createHotModuleAcceptPlugin',
    value: function _createHotModuleAcceptPlugin(entryPointAbsolutePathMap, frameworkConfig) {
      var self = this;
      return function (_ref2) {
        var Plugin = _ref2.Plugin,
            t = _ref2.types;

        return new Plugin("soya-hot-module-accept-plugin", {
          visitor: {}
        });
      };
    }
  }], [{
    key: 'createServerBuildConfig',
    value: function createServerBuildConfig(webpack, frameworkConfig) {
      frameworkConfig = _extends({}, _defaultFrameworkConfig.DEFAULT_FRAMEWORK_CONFIG, frameworkConfig);
      var nodeModules = {};
      var absProjectDir = frameworkConfig.absoluteProjectDir;
      var absEntryPointFile = _path2.default.join(frameworkConfig.absoluteProjectDir, 'build/precompile/server.js');
      var absBuildTargetDir = _path2.default.join(frameworkConfig.absoluteProjectDir, 'build/server');
      _fs2.default.readdirSync(_path2.default.join(absProjectDir, 'node_modules')).filter(function (x) {
        return ['.bin'].indexOf(x) === -1;
      }).forEach(function (mod) {
        nodeModules[mod] = 'commonjs ' + mod;
      });

      return {
        entry: absEntryPointFile,
        target: 'node',
        output: {
          path: absBuildTargetDir,
          publicPath: frameworkConfig.assetProtocol + '://' + frameworkConfig.assetHostPath,
          filename: 'index.js'
        },
        node: {
          __dirname: true,
          __filename: true
        },
        devtool: 'source-map',
        module: {
          loaders: [WebpackCompiler.getBabelLoaderConfig(), WebpackCompiler.getFileLoaderConfig(frameworkConfig), { test: /\.css$/, loader: 'css-loader/locals', exclude: /\.mod\.css/ }, { test: /\.mod\.css$/,
            loader: 'css-loader/locals?' + JSON.stringify({
              sourceMap: true,
              modules: true,
              localIdentName: frameworkConfig.debug ? '[name]_[local]_[hash:base64:3]' : '[hash:base64]'
            })
          }, { test: /\.scss$/, loader: 'css-loader/locals!sass-loader', exclude: /\.mod\.scss/ }, { test: /\.mod\.scss$/, loader: 'css-loader/locals?' + JSON.stringify({
              sourceMap: true,
              modules: true,
              localIdentName: frameworkConfig.debug ? '[name]_[local]_[hash:base64:3]' : '[hash:base64]'
            }) + '!sass-loader' }]
        },
        plugins: [new webpack.BannerPlugin('require("source-map-support").install();', { raw: true, entryOnly: false }), new webpack.optimize.OccurenceOrderPlugin(), new webpack.NoErrorsPlugin()],
        resolve: WebpackCompiler.generateResolveConfig(frameworkConfig),
        externals: nodeModules
      };
    }
  }, {
    key: 'generateResolveConfig',
    value: function generateResolveConfig(frameworkConfig) {
      var absoluteProjectDir = frameworkConfig.absoluteProjectDir,
          defaultImportBase = frameworkConfig.defaultImportBase;


      if (defaultImportBase) {
        defaultImportBase = _path2.default.resolve(absoluteProjectDir, defaultImportBase);
      }

      // Removes undefined config if any
      var rootResolves = [absoluteProjectDir, defaultImportBase].filter(function (config) {
        return !!config;
      });

      var alias = {};
      if (frameworkConfig.routerNodeRegistrationAbsolutePath) {
        alias['soya/lib/server/registerRouterNodes'] = frameworkConfig.routerNodeRegistrationAbsolutePath;
      }

      return {
        alias: alias,
        root: rootResolves
      };
    }

    /**
     * @return {Object}
     */

  }, {
    key: 'getBabelLoaderConfig',
    value: function getBabelLoaderConfig() {
      var result = {
        test: /\.jsx?$/,
        exclude: /(node_modules|bower_components)/,
        loader: 'babel-loader'
      };
      return result;
    }
  }, {
    key: 'getFileLoaderConfig',
    value: function getFileLoaderConfig(config) {
      var test;
      if (config.staticAssetRegex) {
        test = config.staticAssetRegex;
      } else {
        test = /(\.jpg|\.png|\.jpeg|\.gif|\.ico|\.(eot|ttf|svg|woff(2)?)(\??#[a-zA-Z0-9]+)?)$/;
      }
      return {
        test: test,
        loader: "file-loader?name=[name]-[hash].[ext]"
      };
    }
  }]);

  return WebpackCompiler;
}(_Compiler3.default);

exports.default = WebpackCompiler;
//# sourceMappingURL=WebpackCompiler.js.map