'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

exports.default = connect;

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _reactAddonsUpdate = require('react-addons-update');

var _reactAddonsUpdate2 = _interopRequireDefault(_reactAddonsUpdate);

var _helper = require('./helper.js');

var _PropTypes = require('./utils/PropTypes');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/**
 * Wraps a react component inside a component that subscribes to queries to the
 * ReduxStore. The wrapping component encapsulates how, when, what and where
 * to fetch the context. The relationship between a subscriber component and
 * ReduxStore is as follows:
 *
 * 1) Wrapper tells ReduxStore the Segment it needs via getSegmentDependencies().
 * 2) Subscriber component may manually query pieces of Segment it has
 *    registered using each Segment's action creator - it gets action creator
 *    from props provided by wrapper component.
 * 3) Subscriber component may subscribe to a query by implementing
 *    subscribeQueries() static method. This will make wrapper sets query result
 *    as props to the subscriber component. Each change in query result will
 *    trigger re-render of the wrapper (via state), and thus setting of new
 *    props to the subscriber component.
 *
 * Subscriber components props and state relationship:
 *
 * 1) Props are given by owner component to the wrapper component.
 * 2) Wrapper component subscribes to queries and update these subscriptions
 *    based on props provided by owner component.
 * 3) Props in wrapper component determines query in wrapper component.
 * 4) Query subscription determines wrapper component's React state.
 * 5) Wrapper component state determines props to subscriber component.
 *
 * @CLIENT_SERVER
 */

/**
 * This method is called at componentWilMount(). Wrapper will register
 * the returned Segment constructors to ReduxStore. Child components must
 * override this static method to specify their data requirements.
 *
 * @return {Array<Class<Segment>>}
 */
var defaultGetSegmentDependencies = function defaultGetSegmentDependencies() {
  return [];
};

/**
 * Semantics are very similar to shouldComponentUpdate(). This method
 * determines whether to re-run subscribeQueries() or not when this component
 * will receive new props.
 *
 * Props is the prime determiner of query subscription in DataComponent.
 *
 * Default implementation uses shallowEqual. User can override as needed.
 *
 * @param {Object} prevProps
 * @param {Object} nextProps
 *
 * @CLIENT_SERVER
 */
var defaultShouldSubcriptionsUpdate = function defaultShouldSubcriptionsUpdate(prevProps, nextProps) {
  var shouldUpdateSubscription = !(0, _helper.isEqualShallow)(prevProps, nextProps);
  return shouldUpdateSubscription;
};

/**
 * Subscribes to queries to a Segment already registered at
 * getSegmentDependencies().
 *
 * @param {Object} nextProps
 * @param {Function} subscribe
 */
var defaultSubscribeQueries = function defaultSubscribeQueries(nextProps, subscribe) {};

/**
 * TODO: Logger at client! Remove if debug is set to false!
 *
 * @param {React.Component} ReactComponent
 * @param {string} name
 * @return {React.Component}
 */
function connect(ReactComponent) {
  var _class, _temp;

  var connectId = ReactComponent.connectId ? ReactComponent.connectId() : ReactComponent;
  var _getSegmentDependencies = ReactComponent.getSegmentDependencies;
  var subscribeQueries = ReactComponent.subscribeQueries;
  var shouldSubscriptionsUpdate = ReactComponent.shouldSubscriptionsUpdate;
  var shouldWrapperComponentUpdate = ReactComponent.shouldWrapperComponentUpdate;
  if (typeof shouldSubscriptionsUpdate !== 'function') shouldSubscriptionsUpdate = defaultShouldSubcriptionsUpdate;
  if (typeof subscribeQueries !== 'function') subscribeQueries = defaultSubscribeQueries;
  if (typeof _getSegmentDependencies !== 'function') _getSegmentDependencies = defaultGetSegmentDependencies;

  return _temp = _class = function (_React$Component) {
    _inherits(Component, _React$Component);

    _createClass(Component, null, [{
      key: 'getSegmentDependencies',


      /**
       * @type {ReduxStore}
       */


      /**
       * @type {{[key: string]: Function}}
       */


      /**
       * @type {{[key: string]: Object}}
       */
      value: function getSegmentDependencies() {
        return _getSegmentDependencies();
      }

      /**
       * @type {Object}
       */


      /**
       * @type {Function}
       */


      /**
       * @type {Function}
       */

    }]);

    function Component(props, context) {
      _classCallCheck(this, Component);

      // Both actions and unsubscribe is not part of state because it shouldn't
      // affect rendering in any way. Actions are determined at mounting and
      // shouldn't change throughout this component's lifecycle. Unsubscribe
      // contains functions that unsubscribes this component from state changes,
      // but it's the actual state that matters.
      var _this = _possibleConstructorReturn(this, (Component.__proto__ || Object.getPrototypeOf(Component)).call(this, props, context));

      _this.__soyaActions = {};
      _this.__soyaUnsubscribe = {};
      _this.__soyaGetActionCreator = _this.getActionCreator.bind(_this);
      _this.__soyaSubscribe = _this.subscribe.bind(_this);
      _this.__soyaGetReduxStore = _this.getReduxStore.bind(_this);
      _this.__soyaGetConfig = _this.getConfig.bind(_this);
      _this.__soyaContext = props.context || context.context;

      var reduxStore = _this.getReduxStore();
      var config = _this.getConfig();
      if (!reduxStore || !reduxStore.__isReduxStore) {
        throw new Error('ReduxStore is not properly wired to this data component: ' + ReactComponent + '.');
      }
      if (!config || (typeof config === 'undefined' ? 'undefined' : _typeof(config)) != 'object') {
        throw new Error('Config object is not properly wired to this data component: ' + ReactComponent + '.');
      }
      return _this;
    }

    /**
     * Defaults to getting ReduxStore instance from props.store. If you want to
     * use context you can override this method.
     *
     * @returns {ReduxStore}
     */


    _createClass(Component, [{
      key: 'getReduxStore',
      value: function getReduxStore() {
        if (this.__soyaContext && this.__soyaContext.store) {
          return this.__soyaContext.store;
        }
        if (this.__soyaContext && this.__soyaContext.reduxStore) {
          console.warn('Please use Page.createContext() to create context instead of creating it by yourself!');
          return this.__soyaContext.reduxStore;
        }
        throw new Error('Context not properly wired to: ' + connectId + '.');
      }

      /**
       * Defaults to getting Config object from props.context. If you want to use
       * context you can override this method.
       *
       * @returns {Object}
       */

    }, {
      key: 'getConfig',
      value: function getConfig() {
        if (this.__soyaContext && this.__soyaContext.config) {
          return this.__soyaContext.config;
        }
        throw new Error('Context not properly wired to: ' + connectId + '.');
      }

      /**
       * @param {string} segmentId
       * @param {any} query
       * @param {string} stateName
       * @param {Hydration} hydration
       */

    }, {
      key: 'subscribe',
      value: function subscribe(segmentId, query, stateName, hydration) {
        var _this2 = this;

        // Unsubscribe if already subscribed.
        this.unsubscribe(stateName);

        //console.log('[SUB] Subscribe', connectId, segmentId);
        var callback = function callback(newState) {
          _this2.setState(_defineProperty({}, stateName, newState));
        };

        var storeRef = this.getReduxStore().subscribe(segmentId, query, callback, this, hydration);
        this.unsubscribe[segmentId] = storeRef.unsubscribe;
        this.setState(_defineProperty({}, stateName, storeRef.getState()));
      }

      /**
       * @param {string} stateName
       */

    }, {
      key: 'unsubscribe',
      value: function unsubscribe(stateName) {
        if (this.__soyaUnsubscribe[stateName]) {
          //console.log('[SUB] Un-subscribe', connectId, stateName);
          this.__soyaUnsubscribe[stateName]();
          delete this.__soyaUnsubscribe[stateName];
        }
      }

      /**
       * Returns action creator of an already registered segment.
       *
       * @param {string} segmentId
       * @return {Object}
       */

    }, {
      key: 'getActionCreator',
      value: function getActionCreator(segmentId) {
        if (!this.__soyaActions.hasOwnProperty(segmentId)) {
          throw new Error('Unable to get action creator for segment \'' + segmentId + '\', segment is not registered.');
        }
        return this.__soyaActions[segmentId];
      }
    }, {
      key: 'connectedProps',
      value: function connectedProps(props) {
        var nextProps = (0, _reactAddonsUpdate2.default)(props, { getActionCreator: { $set: this.__soyaGetActionCreator } });
        nextProps.context = this.__soyaContext;
        nextProps.result = this.state;
        nextProps.getReduxStore = this.__soyaGetReduxStore;
        nextProps.getConfig = this.__soyaGetConfig;
        return nextProps;
      }
    }, {
      key: 'render',
      value: function render() {
        if (!this.__soyaGetActionCreator) this.__soyaGetActionCreator = this.getActionCreator.bind(this);
        return _react2.default.createElement(ReactComponent, this.connectedProps(this.props));
      }

      /**
       * @param {Class<Segment>} segmentClass
       */

    }, {
      key: '_register',
      value: function _register(segmentClass) {
        var actionCreator = this.getReduxStore().register(segmentClass);
        this.__soyaActions[segmentClass.id()] = actionCreator;
      }

      /**
       * This implementation assumes:
       *
       * 1) All subscriber components render only using props and states (i.e.
       *    they are pure render components). Child classes can override this
       *    method if they are not.
       * 2) All wrapper component owner send their props as immutable objects
       *    (i.e. if the prop changes, they recreate the whole object).
       *
       * Based on the above assumptions, this method will do a shallow comparison
       * of both state and props. Child component can stop re-rendering by
       * creating their own shouldComponentUpdate method.
       *
       * @param {Object} nextProps
       * @param {Object} nextState
       * @return {boolean}
       */

    }, {
      key: 'shouldComponentUpdate',
      value: function shouldComponentUpdate(nextProps, nextState) {
        var connectedProps = this.connectedProps(this.props);
        var nextConnectedProps = this.connectedProps(nextProps);
        if (shouldWrapperComponentUpdate) {
          return shouldWrapperComponentUpdate(connectedProps, nextConnectedProps, this.state, nextState);
        }
        var shouldUpdate = !(0, _helper.isEqualShallow)(connectedProps, nextConnectedProps) || !(0, _helper.isEqualShallow)(this.state, nextState);
        console.log('[SUB] Should update?', connectId, shouldUpdate);
        return shouldUpdate;
      }
    }, {
      key: 'componentWillUpdate',
      value: function componentWillUpdate(nextProps, nextState) {}
      //console.log('[SUB] Will update', connectId, nextProps, nextState);


      /**
       * @param {Object} nextProps
       */

    }, {
      key: 'componentWillReceiveProps',
      value: function componentWillReceiveProps(nextProps) {
        var connectedProps = this.connectedProps(this.props);
        var nextConnectedProps = this.connectedProps(nextProps);
        var shouldUpdateSubscriptions = shouldSubscriptionsUpdate(connectedProps, nextConnectedProps);
        console.log('[SUB] Should subscriptions update', connectId, shouldUpdateSubscriptions);
        if (shouldUpdateSubscriptions) {
          // If query subscriptions update, we need to remove past subscriptions.
          // Otherwise this component may unnecessarily re-render each time the
          // already unrelated segment piece changes. Since this is costly, this
          // is why shouldSubscriptionsUpdate() is important.
          this.getReduxStore().unsubscribe(this);
          subscribeQueries(nextConnectedProps, this.__soyaSubscribe);
        }
      }

      /**
       * Registers the store. Run at both client and server side when rendering.
       */

    }, {
      key: 'componentWillMount',
      value: function componentWillMount() {
        //console.log('[SUB] Mounting', connectId, this.props);
        var i,
            segmentClasses = _getSegmentDependencies();
        for (i = 0; i < segmentClasses.length; i++) {
          this._register(segmentClasses[i]);
        }
        subscribeQueries(this.connectedProps(this.props), this.__soyaSubscribe);
      }

      /**
       * Unsubscribe all segments.
       */

    }, {
      key: 'componentWillUnmount',
      value: function componentWillUnmount() {
        //console.log('[SUB] Unmounting', connectId);
        this.getReduxStore().unsubscribe(this);
      }
    }]);

    return Component;
  }(_react2.default.Component), _class.contextTypes = {
    context: _PropTypes.contextShape
  }, _temp;
}
//# sourceMappingURL=connect.js.map