'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

exports.default = createField;

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _helper = require('../helper.js');

var _PromiseUtil = require('../PromiseUtil.js');

var _PromiseUtil2 = _interopRequireDefault(_PromiseUtil);

var _FormSegment = require('./FormSegment.js');

var _FormSegment2 = _interopRequireDefault(_FormSegment);

var _connect = require('../connect.js');

var _connect2 = _interopRequireDefault(_connect);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/**
 * Wraps the given InputComponent in a component that links to FormSegment with
 * the following contract:
 *
 * 1) InputComponent must get value from props.value when rendering.
 * 2) InputComponent must run props.handleChange when the value changes.
 * 3) InputComponent may run props.handleAsyncChange if it wants to be able to
 *    run async validation and async change handlers.
 *
 * Accepts the following special props:
 *
 * 1) reduxStore
 * 2) config
 * 3) changeHandlers (array of functions, optional)
 * 4) changeValidators (array of functions, optional)
 * 5) asyncValidators (array of functions, optional)
 * 6) submitValidators (array of functions, optional)
 *
 * Any other props will be passed on to the wrapped input component untouched.
 *
 * @param {React.Component} InputComponent
 * @return {ReactComponent}
 *
 * @CLIENT_SERVER
 */
function createField(InputComponent) {
  var type = typeof InputComponent.getQueryType == 'function' ? InputComponent.getQueryType() : 'field';

  // TODO: Move these methods into separate functions above, so that they are reused instead of always recreated.

  var Component = function (_React$Component) {
    _inherits(Component, _React$Component);

    _createClass(Component, null, [{
      key: 'connectId',


      /**
       * @returns {string}
       */
      value: function connectId() {
        return InputComponent.connectId ? InputComponent.connectId() : 'Field Component';
      }

      /**
       * @returns {Array<Segment>}
       */

    }, {
      key: 'getSegmentDependencies',
      value: function getSegmentDependencies() {
        return [_FormSegment2.default];
      }
    }, {
      key: 'shouldSubscriptionsUpdate',
      value: function shouldSubscriptionsUpdate(props, nextProps) {
        var isNameString = (0, _helper.isStringDuckType)(props.name);
        var isNextNameString = (0, _helper.isStringDuckType)(nextProps.name);
        return props.form !== nextProps.form || isNameString !== isNextNameString || isNameString && props.name !== nextProps.name || !isNameString && !(0, _helper.isEqualShallowArray)(props.name, nextProps.name);
      }
    }, {
      key: 'subscribeQueries',
      value: function subscribeQueries(props, subscribe) {
        subscribe(_FormSegment2.default.id(), {
          formId: props.form.getFormId(),
          type: type,
          fieldName: props.name
        }, 'field');
        subscribe(_FormSegment2.default.id(), {
          formId: props.form.getFormId(),
          type: 'isEnabled'
        }, 'isEnabled');
        subscribe(_FormSegment2.default.id(), {
          formId: props.form.getFormId(),
          type: 'isSubmitting'
        }, 'isSubmitting');
      }
    }]);

    function Component(props, context) {
      _classCallCheck(this, Component);

      var _this = _possibleConstructorReturn(this, (Component.__proto__ || Object.getPrototypeOf(Component)).call(this, props, context));

      _this.__inputChangeValidators = [];
      _this.__inputAsyncValidators = [];
      _this.__submitValidators = [];
      _this.__setDefaultValue = _this.setDefaultValue.bind(_this);
      _this.__handleChange = _this.handleChange.bind(_this);
      _this.__handleAsyncValidation = _this.handleAsyncValidation.bind(_this);
      _this.__registerAsyncValidators = _this.registerAsyncValidators.bind(_this);
      _this.__registerChangeValidators = _this.registerChangeValidators.bind(_this);
      _this.__registerSubmitValidators = _this.registerSubmitValidators.bind(_this);
      return _this;
    }

    _createClass(Component, [{
      key: 'componentWillMount',
      value: function componentWillMount() {
        // TODO: Also do this when component receive new props, since owner component may reassign this component to a new field.
        this.props.form.regField(this.props.name, this.handleValidateAll.bind(this));
      }
    }, {
      key: 'componentWillUnmount',
      value: function componentWillUnmount() {
        this.props.form.unregField(this.props.name);
      }
    }, {
      key: 'shouldComponentUpdate',
      value: function shouldComponentUpdate(nextProps, nextState) {
        // We don't use state, so we don't have to check it. But we do have to
        // check for props.
        return !(0, _helper.isEqualShallow)(nextProps, this.props);
      }
    }, {
      key: 'render',
      value: function render() {
        // Pass appropriate props to the InputComponent.
        var key,
            props = {};
        for (key in this.props) {
          if (!this.props.hasOwnProperty(key)) continue;
          if (key == 'result' || key == 'changeHandlers' || key == 'validators' || key == 'validatorsAsync') continue;
          props[key] = this.props[key];
        }

        if (this.props.result.field) {
          props.value = this.props.result.field.value;
          props.errorMessages = this.props.result.field.errorMessages;
          props.touched = this.props.result.field.touched;
          props.isValidating = this.props.result.field.isValidating;
          props.isFieldEnabled = this.props.result.field.isEnabled;
        } else {
          // TODO: Use default value from FormSegment.
          props.value = null;
          props.errorMessages = [];
          props.touched = false;
          props.isFieldEnabled = true;
          props.isValidating = false;
        }

        if (this.props.result.hasOwnProperty('isEnabled')) {
          props.isFormEnabled = this.props.result.isEnabled;
        } else {
          props.isFormEnabled = true;
        }

        if (this.props.result.hasOwnProperty('isSubmitting')) {
          props.isFormSubmitting = this.props.result.isSubmitting;
        } else {
          props.isFormSubmitting = false;
        }

        props.isDisabled = props.isValidating || !props.isFormEnabled || !props.isFieldEnabled || props.isFormSubmitting;
        props.setDefaultValue = this.__setDefaultValue;
        props.handleChange = this.__handleChange;
        props.handleAsyncValidation = this.__handleAsyncValidation;
        props.registerChangeValidators = this.__registerChangeValidators;
        props.registerAsyncValidators = this.__registerAsyncValidators;
        props.registerSubmitValidators = this.__registerSubmitValidators;

        // For performance optimizations, InputComponent may implement
        // shouldComponentUpdate that checks for changes in props.
        return _react2.default.createElement(InputComponent, _extends({ key: 'main' }, props));
      }

      /**
       * @param {Array<Function>} funcArray
       */

    }, {
      key: 'registerChangeValidators',
      value: function registerChangeValidators(funcArray) {
        this.__inputChangeValidators = this.__inputChangeValidators.concat(funcArray);
      }

      /**
       * @param {Array<Function>} funcArray
       */

    }, {
      key: 'registerAsyncValidators',
      value: function registerAsyncValidators(funcArray) {
        this.__inputAsyncValidators = this.__inputAsyncValidators.concat(funcArray);
      }

      /**
       * @param {Array<Function>} funcArray
       */

    }, {
      key: 'registerSubmitValidators',
      value: function registerSubmitValidators(funcArray) {
        this.__submitValidators = this.__submitValidators.concat(funcArray);
      }
    }, {
      key: 'setDefaultValue',
      value: function setDefaultValue(value) {
        var actions = _FormSegment2.default.getActionCreator();
        this.props.getReduxStore().dispatch(actions.setDefaultValue(this.props.form.getFormId(), this.props.name, value));
      }

      /**
       * @param {?} value
       */

    }, {
      key: 'handleChange',
      value: function handleChange(value) {
        // TODO: Change usage this.props.getActionCreator() to FormSegment.getActionCreator().
        var i,
            errorMessages = this.validateSync(value);
        var actions = this.props.getActionCreator(_FormSegment2.default.id());
        if (errorMessages.length == 0) {
          this.props.getReduxStore().dispatch(actions.setValue(this.props.form.getFormId(), this.props.name, value));
        } else {
          this.props.getReduxStore().dispatch(actions.mergeFields(this.props.form.getFormId(), [{
            fieldName: this.props.name,
            object: {
              value: value,
              touched: true,
              errorMessages: errorMessages
            }
          }]));
        }

        // After dispatching (and closing the loop). Run the change handlers.
        if (this.props.changeHandlers) {
          for (i = 0; i < this.props.changeHandlers.length; i++) {
            this.props.changeHandlers[i](value);
          }
        }
      }

      /**
       * Runs all sync and async validators. Returns a promise that resolves to
       * the following object:
       *
       * <pre>
       *   {
       *     isValid: true/false,
       *     name: (field name),
       *     value: (the value)
       *   }
       * </pre>
       *
       * The promise should reject if there's an error with async validation.
       *
       * @returns {Promise}
       */

    }, {
      key: 'handleValidateAll',
      value: function handleValidateAll() {
        var _this2 = this;

        var name = this.props.name;
        var value = this.props.result.field ? this.props.result.field.value : null;
        var errorMessages = this.validateSync(value);
        if (errorMessages.length > 0) {
          var actions = this.props.getActionCreator(_FormSegment2.default.id());
          this.props.getReduxStore().dispatch(actions.mergeFields(this.props.form.getFormId(), [{
            fieldName: this.props.name,
            object: {
              errorMessages: errorMessages
            }
          }]));
          return Promise.resolve({ isValid: false, value: value, name: name });
        }

        var asyncPromise, submitPromise;
        var hasAsyncValidation = this.hasAsyncValidation();
        var hasSubmitValidation = this.hasSubmitValidation();
        if (!hasAsyncValidation && !hasSubmitValidation) {
          return Promise.resolve({ isValid: true, value: value, name: name });
        }

        if (!hasAsyncValidation) {
          asyncPromise = Promise.resolve(true);
        } else {
          this.lock();
          asyncPromise = this.validateAsync(value);
        }

        if (!hasSubmitValidation) {
          submitPromise = Promise.resolve(true);
        } else {
          submitPromise = this.validateSubmit(value);
        }

        return _PromiseUtil2.default.allParallel(Promise, [asyncPromise, submitPromise]).then(function (results) {
          _this2.unlock();
          var isValid = (0, _helper.mergeValidationResult)(results);
          return { isValid: isValid, value: value, name: name };
        }).catch(function (error) {
          _this2.unlock();
          _PromiseUtil2.default.throwError(error);
        });
      }

      /**
       * This handler doesn't update value - we assume that handleChange
       * is already run and thus value in FormSegment is already updated. It's
       * a trigger for asynchronous validation. We assume that:
       *
       * - Asynchronous validation must trigger on a less often triggered event,
       *   like onBlur or a predetermined timeout.
       * - The input component will read isValidating value and appropriately
       *   disables the input when the asynchronous validation is running to
       *   prevent lagging async validation to happen.
       *
       * Returns a promise that resolves to true if the value passes validation,
       * or false if not.
       *
       * @return {?Promise}
       */

    }, {
      key: 'handleAsyncValidation',
      value: function handleAsyncValidation() {
        var _this3 = this;

        var value = this.props.result.field ? this.props.result.field.value : null;
        var errorMessages = this.validateSync(value);
        if (errorMessages.length > 0) {
          // No need to continue if sync validation has failed.
          return Promise.resolve(false);
        }

        // No need to run if we have no async validation.
        var hasAsyncValidation = this.hasAsyncValidation();
        if (!hasAsyncValidation) {
          return Promise.resolve(true);
        }

        this.lock();
        var promise = this.validateAsync(value);
        promise.then(this.unlock.bind(this)).catch(function (error) {
          _this3.unlock();
          _PromiseUtil2.default.throwError(error);
        });
        return promise;
      }

      /**
       * Runs input and props sync validators on the value and returns the error
       * messages.
       *
       * @param {?} value
       * @return {Array<string>}
       */

    }, {
      key: 'validateSync',
      value: function validateSync(value) {
        // Run validation, run user validation first since it may contain the
        // required or optional validator.
        var i,
            result,
            errorMessages = [];
        if (this.props.changeValidators) {
          for (i = 0; i < this.props.changeValidators.length; i++) {
            result = this.props.changeValidators[i](value);
            if (!this.pushErrorMessage(result, errorMessages)) return errorMessages;
          }
        }
        for (i = 0; i < this.__inputChangeValidators.length; i++) {
          result = this.__inputChangeValidators[i](value);
          if (!this.pushErrorMessage(result, errorMessages)) return errorMessages;
        }
        return errorMessages;
      }

      /**
       * Returns a promise that resolves to true if value passes validation, or
       * false if not.
       *
       * @param {?} value
       * @returns {Promise}
       */

    }, {
      key: 'validateAsync',
      value: function validateAsync(value) {
        var i,
            promises = [];
        for (i = 0; i < this.__inputAsyncValidators.length; i++) {
          promises.push(this.__inputAsyncValidators[i](value));
        }
        if (this.props.asyncValidators) {
          for (i = 0; i < this.props.asyncValidators.length; i++) {
            promises.push(this.props.asyncValidators[i](value));
          }
        }
        return this._createValidatorPromise(promises);
      }

      /**
       * @param {?} value
       * @return {Promise}
       */

    }, {
      key: 'validateSubmit',
      value: function validateSubmit(value) {
        var i,
            promises = [];
        for (i = 0; i < this.__submitValidators.length; i++) {
          promises.push(Promise.resolve(this.__submitValidators[i](value)));
        }
        if (this.props.submitValidators) {
          for (i = 0; i < this.props.submitValidators.length; i++) {
            promises.push(Promise.resolve(this.props.submitValidators[i](value)));
          }
        }
        return this._createValidatorPromise(promises);
      }

      /**
       * @param {Array<Promise>} promises
       * @returns {Promise}
       * @private
       */

    }, {
      key: '_createValidatorPromise',
      value: function _createValidatorPromise(promises) {
        var _this4 = this;

        var parallelPromise = _PromiseUtil2.default.allParallel(Promise, promises);
        var finalPromise = new Promise(function (resolve, reject) {
          parallelPromise.then(function (result) {
            var i,
                errorMessages = [];
            var actions = _this4.props.getActionCreator(_FormSegment2.default.id());
            for (i = 0; i < result.length; i++) {
              if (typeof result[i] == 'string') errorMessages.push(result[i]);
            }
            _this4.props.getReduxStore().dispatch(actions.addErrorMessages(_this4.props.form.getFormId(), [{
              fieldName: _this4.props.name,
              messages: errorMessages
            }]));
            resolve(errorMessages.length <= 0);
          }, function (error) {
            reject(error);
          });
        });
        return finalPromise;
      }

      /**
       * @returns {boolean}
       */

    }, {
      key: 'hasAsyncValidation',
      value: function hasAsyncValidation() {
        return this.__inputAsyncValidators.length > 0 || this.props.asyncValidators && this.props.asyncValidators.length > 0;
      }

      /**
       * @return {boolean}
       */

    }, {
      key: 'hasSubmitValidation',
      value: function hasSubmitValidation() {
        return this.__submitValidators.length > 0 || this.props.submitValidators && this.props.submitValidators.length > 0;
      }

      /**
       * Locks this field by setting its isValidating flag to true.
       */

    }, {
      key: 'lock',
      value: function lock() {
        var actions = this.props.getActionCreator(_FormSegment2.default.id());
        this.props.getReduxStore().dispatch(actions.setIsValidating(this.props.form.getFormId(), _defineProperty({}, this.props.name, true)));
      }

      /**
       * Unlock this field by setting its isValidating flag to false.
       */

    }, {
      key: 'unlock',
      value: function unlock() {
        var actions = this.props.getActionCreator(_FormSegment2.default.id());
        this.props.getReduxStore().dispatch(actions.setIsValidating(this.props.form.getFormId(), _defineProperty({}, this.props.name, false)));
      }

      /**
       * Returns true if next validator function should be called. Returns false
       * otherwise.
       *
       * @param {string|boolean} result
       * @param {Array<string>} errorMessages
       * @return {boolean}
       */

    }, {
      key: 'pushErrorMessage',
      value: function pushErrorMessage(result, errorMessages) {
        if (result === null) return false;
        if (typeof result === 'string' || (typeof result === 'undefined' ? 'undefined' : _typeof(result)) === 'object') errorMessages.push(result);
        return true;
      }
    }]);

    return Component;
  }(_react2.default.Component);

  return (0, _connect2.default)(Component);
}
//# sourceMappingURL=createField.js.map