'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

exports.default = createRepeatable;

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _helper = require('../helper.js');

var _PromiseUtil = require('../PromiseUtil.js');

var _PromiseUtil2 = _interopRequireDefault(_PromiseUtil);

var _FormSegment = require('./FormSegment.js');

var _FormSegment2 = _interopRequireDefault(_FormSegment);

var _connect = require('../connect.js');

var _connect2 = _interopRequireDefault(_connect);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/**
 * This function will wrap the given react component into a Repeatable
 * component. Repeatable component will render the given component as many as
 * the length of the list pointed out by the name props.
 *
 * Wrapper will provide special function props, along with essential reduxStore
 * and config to the field set component.
 *
 * @param {React.Component} FieldSetComponent
 * @return {React.Component}
 *
 * @CLIENT_SERVER
 */
function createRepeatable(FieldSetComponent) {
  // TODO: Move each method into separate functions above so they are reused instead of being recreated every call.
  var Component = function (_React$Component) {
    _inherits(Component, _React$Component);

    _createClass(Component, null, [{
      key: 'connectId',
      value: function connectId() {
        return FieldSetComponent.connectId ? FieldSetComponent.connectId() : 'Repeatable Field Component';
      }
    }, {
      key: 'getSegmentDependencies',
      value: function getSegmentDependencies() {
        return [_FormSegment2.default];
      }
    }, {
      key: 'subscribeQueries',
      value: function subscribeQueries(props, subscribe) {
        subscribe(_FormSegment2.default.id(), {
          formId: props.form._formId,
          type: 'length',
          fieldName: props.name,
          minLength: props.minLength
        }, 'length');
      }
    }, {
      key: 'shouldSubscriptionsUpdate',
      value: function shouldSubscriptionsUpdate(props, nextProps) {
        return props.form !== nextProps.form || props.name !== nextProps.name || props.length !== nextProps.length || props.minLength !== nextProps.minLength || props.maxLength !== nextProps.maxLength;
      }
    }]);

    function Component(props, context) {
      _classCallCheck(this, Component);

      var _this = _possibleConstructorReturn(this, (Component.__proto__ || Object.getPrototypeOf(Component)).call(this, props, context));

      _this._nameFuncCache = [];
      _this._addListItemFunc = _this.addListItem.bind(_this);
      _this._removeListItemFunc = _this.removeListItem.bind(_this);
      _this._reorderListItemIncFunc = _this.reorderListItemInc.bind(_this);
      _this._reorderListItemDecFunc = _this.reorderListItemDec.bind(_this);
      _this._reorderListItemFunc = _this.reorderListItem.bind(_this);
      return _this;
    }

    _createClass(Component, [{
      key: 'componentWillMount',
      value: function componentWillMount() {
        this._baseName = (0, _helper.isArray)(this.props.name) ? this.props.name : [this.props.name];
      }
    }, {
      key: 'componentWillReceiveProps',
      value: function componentWillReceiveProps(nextProps) {
        this._baseName = (0, _helper.isArray)(nextProps.name) ? nextProps.name : [nextProps.name];
      }
    }, {
      key: 'shouldComponentUpdate',
      value: function shouldComponentUpdate(nextProps, nextState) {
        // We don't use state, so we don't have to check it. But we do have to
        // check for props.
        return !(0, _helper.isEqualShallow)(nextProps, this.props);
      }
    }, {
      key: 'render',
      value: function render() {
        var i,
            props = {},
            fieldSetList = null;
        var minLength = this.props.minLength || 0;
        var length = this.props.result.length > minLength ? this.props.result.length : minLength;
        if (length > 0) {
          for (i in this.props) {
            if (!this.props.hasOwnProperty(i)) continue;
            props[i] = this.props[i];
          }
          fieldSetList = [];
          props.totalItem = length;
          props.addListItem = this._addListItemFunc;
          props.removeListItem = this._removeListItemFunc;
          props.reorderListItemInc = this._reorderListItemIncFunc;
          props.reorderListItemDec = this._reorderListItemDecFunc;
          props.reorderListItem = this._reorderListItemFunc;
          for (i = 0; i < length; i++) {
            props.index = i;
            props.name = this.getNameFunc(i);
            fieldSetList.push(_react2.default.createElement(FieldSetComponent, _extends({ key: i }, props)));
          }
        }
        return _react2.default.createElement(
          'div',
          null,
          fieldSetList
        );
      }

      /**
       * @param {number} index
       * @param {Array<string>|string} fieldName
       * @returns {Array<string>}
       */

    }, {
      key: 'name',
      value: function name(index, fieldName) {
        return this._baseName.concat(index).concat(fieldName);
      }

      /**
       * @param {number} index
       */

    }, {
      key: 'removeListItem',
      value: function removeListItem(index) {
        var actions = this.props.getActionCreator(_FormSegment2.default.id());
        this.props.getReduxStore().dispatch(actions.removeListItem(this.props.form._formId, this.props.name, index));
      }
    }, {
      key: 'addListItem',
      value: function addListItem() {
        var actions = this.props.getActionCreator(_FormSegment2.default.id());
        this.props.getReduxStore().dispatch(actions.addListItem(this.props.form._formId, this.props.name, this.props.minLength, this.props.maxLength));
      }

      /**
       * @param {number} index
       * @param {number} amount
       */

    }, {
      key: 'reorderListItemInc',
      value: function reorderListItemInc(index, amount) {
        var actions = this.props.getActionCreator(_FormSegment2.default.id());
        this.props.getReduxStore().dispatch(actions.reorderListItemInc(this.props.form._formId, this.props.name, index, amount));
      }

      /**
       * @param {number} index
       * @param {number} amount
       */

    }, {
      key: 'reorderListItemDec',
      value: function reorderListItemDec(index, amount) {
        var actions = this.props.getActionCreator(_FormSegment2.default.id());
        this.props.getReduxStore().dispatch(actions.reorderListItemDec(this.props.form._formId, this.props.name, index, amount));
      }

      /**
       * @param {number} index
       * @param {number} targetIndex
       */

    }, {
      key: 'reorderListItem',
      value: function reorderListItem(index, targetIndex) {
        var actions = this.props.getActionCreator(_FormSegment2.default.id());
        this.props.getReduxStore().dispatch(actions.reorderListItem(this.props.form._formId, this.props.name, index, targetIndex));
      }
    }, {
      key: 'getNameFunc',
      value: function getNameFunc(index) {
        if (!this._nameFuncCache.length.hasOwnProperty(index)) {
          this._nameFuncCache[index] = this.name.bind(this, index);
        }
        return this._nameFuncCache[index];
      }
    }]);

    return Component;
  }(_react2.default.Component);

  return (0, _connect2.default)(Component);
};
//# sourceMappingURL=createRepeatable.js.map