'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _CookieJar2 = require('./CookieJar.js');

var _CookieJar3 = _interopRequireDefault(_CookieJar2);

var _Cookie = require('./Cookie.js');

var _Cookie2 = _interopRequireDefault(_Cookie);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/**
 * @SERVER
 */
var ServerCookieJar = function (_CookieJar) {
  _inherits(ServerCookieJar, _CookieJar);

  /**
   * @param {http.incomingMessage} httpRequest
   */


  /**
   * @type {{[key: string]: string}}
   */
  function ServerCookieJar(httpRequest) {
    _classCallCheck(this, ServerCookieJar);

    var _this = _possibleConstructorReturn(this, (ServerCookieJar.__proto__ || Object.getPrototypeOf(ServerCookieJar)).call(this));

    _this._httpRequest = httpRequest;
    _this._newCookies = {};
    return _this;
  }

  /**
   * TODO: Reading cookie might also need domain?
   *
   * @param {string} name
   * @return {?string}
   */


  /**
   * @type {{[key: string]: Cookie}}
   */

  /**
   * @type {http.incomingMessage}
   */


  _createClass(ServerCookieJar, [{
    key: 'read',
    value: function read(name) {
      // Get from new cookies first, they represent the *latest* cookie state.
      var newCookie = this._newCookies[name];
      if (newCookie) {
        return newCookie.value;
      }

      // Otherwise try to get it from http request.
      if (this._cookies == null) {
        this._createCache();
      }
      if (this._cookies.hasOwnProperty(name)) {
        return this._cookies[name];
      }
      return null;
    }

    /**
     * @param {Cookie} cookie
     */

  }, {
    key: 'set',
    value: function set(cookie) {
      if (!(cookie instanceof _Cookie2.default)) {
        throw new Error('Expected cookie, this given instead: ' + cookie);
      }
      this._newCookies[cookie.name] = cookie;
    }

    /**
     * @param {string} name
     * @param {string} domain
     * @param {boolean} secure
     * @param {?string} path
     */

  }, {
    key: 'remove',
    value: function remove(name, domain, secure, path) {
      var removalCookie = _Cookie2.default.createRemoval(name, domain, secure, path);
      this.set(removalCookie);
    }

    /**
     * Generates an array of 'Set-Cookie' header values.
     *
     * @return {Array<string>}
     */

  }, {
    key: 'generateHeaderValues',
    value: function generateHeaderValues() {
      var key,
          values = [];
      for (key in this._newCookies) {
        values.push(this._newCookies[key].toHeaderString());
      }
      return values;
    }
  }, {
    key: 'getHttpRequest',
    value: function getHttpRequest() {
      return this._httpRequest;
    }

    /**
     * Creates a cache of cookie from http.incomingMessage.
     */

  }, {
    key: '_createCache',
    value: function _createCache() {
      this._cookies = {};
      var cookieString = this._httpRequest.headers.cookie;
      if (cookieString == null || typeof cookieString != 'string') return null;
      var splitCookieStr = cookieString.split(';');
      // TODO: Security risk, parsing cookies here means blocking the event loop. If attacker sends large amount of cookies, it could make our server non-responsive.
      for (var i = 0; i < splitCookieStr.length; i++) {
        var cookieSegment = splitCookieStr[i];
        while (cookieSegment.charAt(0) == ' ') {
          cookieSegment = cookieSegment.substring(1, cookieSegment.length);
        }
        // Since request cookies don't change, we can cache it.
        var equalityIndex = cookieSegment.indexOf('=');
        var cookieName = cookieSegment.substring(0, equalityIndex);
        var cookieVal = cookieSegment.substring(equalityIndex + 1, cookieSegment.length);
        this._cookies[cookieName] = cookieVal;
      }
    }
  }]);

  return ServerCookieJar;
}(_CookieJar3.default);

exports.default = ServerCookieJar;
//# sourceMappingURL=ServerCookieJar.js.map