'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _Node2 = require('./Node');

var _Node3 = _interopRequireDefault(_Node2);

var _EmptyPathNode = require('./EmptyPathNode');

var _EmptyPathNode2 = _interopRequireDefault(_EmptyPathNode);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/**
 * @CLIENT-SERVER
 */
var PathNode = function (_Node) {
  _inherits(PathNode, _Node);

  _createClass(PathNode, null, [{
    key: 'constructFromConfig',


    /**
     * @param {Array<string>} argArray
     * @return {Array<Node>}
     */
    value: function constructFromConfig(argArray) {
      if (argArray.length != 1) {
        throw new Error('Domain node only receives one argument! Got ' + argArray.length + '.');
      }
      return PathNode.createFromPath(argArray[0] + '');
    }

    /**
     * Create list of PathNode representing the given path.
     *
     * @param {string} path
     * @return {Array<PathNode>}
     */

  }, {
    key: 'createFromPath',
    value: function createFromPath(path) {
      var i,
          result = [],
          pathSegments = PathNode.toPathSegments(path);
      for (i = 0; i < pathSegments.length; i++) {
        result.push(new PathNode(pathSegments[i]));
      }
      if (result.length == 0) {
        // Path is empty, assume it's for home page.
        result.push(new _EmptyPathNode2.default());
      }
      return result;
    }

    /**
     * Split path into array of segments.
     *
     * '/' --> []
     * '/foo/bar' --> ['foo', 'bar']
     * '/foo/bar/' --> ['foo', 'bar', '']
     *
     * @param {string} path
     * @return {Array<string>}
     */

  }, {
    key: 'toPathSegments',
    value: function toPathSegments(path) {
      var splitPath = path.split('/');

      // Slash prefix is not useful at routing, as it always exists.
      // Slash at the end, however, is necessary for routing, so we want to keep that.
      if (splitPath[0] == '') {
        splitPath = splitPath.slice(1);
      }

      return splitPath;
    }

    /**
     * @type {string}
     */

  }, {
    key: 'order',

    /**
     * @returns {number}
     */
    get: function get() {
      return 30;
    }

    /**
     * @returns {string}
     */

  }, {
    key: 'nodeName',
    get: function get() {
      return 'path';
    }
  }]);

  /**
   * @param {string} segment
   */
  function PathNode(segment) {
    _classCallCheck(this, PathNode);

    var _this = _possibleConstructorReturn(this, (PathNode.__proto__ || Object.getPrototypeOf(PathNode)).call(this));

    _this._segment = segment;
    return _this;
  }

  /**
   * @param {RoutingData} routingData
   */


  _createClass(PathNode, [{
    key: 'evaluate',
    value: function evaluate(routingData) {
      if (routingData.isAllSegmentConsumed()) {
        // We are still expecting a path segment. If all path segment is already
        // consumed, it's not a match.
        return false;
      }

      var isMatch = false;
      var decodedCurrentSegment = decodeURIComponent(routingData.getCurrentSegment());
      if (this._segment[0] == ':') {
        // Match all, but set route args first.
        var varName = this._segment.substr(1);
        routingData.resultRouteArgs[varName] = decodedCurrentSegment;
        isMatch = true;
      } else if (this._segment == decodedCurrentSegment) {
        // Else match it char by char.
        isMatch = true;
      }

      if (isMatch) {
        routingData.consumeSegment();
        return true;
      }
      return false;
    }

    /**
     * @param {ReverseRoutingData} reverseRoutingData
     */

  }, {
    key: 'reverseEvaluate',
    value: function reverseEvaluate(reverseRoutingData) {
      if (this._segment[0] == ':') {
        var varName = this._segment.substr(1);
        reverseRoutingData.pathSegments.push(encodeURIComponent(reverseRoutingData.routeArgs[varName]));
        return;
      }
      reverseRoutingData.pathSegments.push(encodeURIComponent(this._segment));
    }

    /**
     * @returns {boolean}
     */

  }, {
    key: 'isConsumingSegmentOnMatch',
    value: function isConsumingSegmentOnMatch() {
      return true;
    }

    /**
     * @param {Node} node
     * @returns {boolean}
     */

  }, {
    key: 'equals',
    value: function equals(node) {
      return node instanceof PathNode && node._segment == this._segment;
    }
  }]);

  return PathNode;
}(_Node3.default);

exports.default = PathNode;
//# sourceMappingURL=PathNode.js.map