'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

// These dependencies can all be overwritten by user.


exports.default = server;

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _jsYaml = require('js-yaml');

var _jsYaml2 = _interopRequireDefault(_jsYaml);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _webpack = require('webpack');

var _webpack2 = _interopRequireDefault(_webpack);

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _Router = require('../router/Router.js');

var _Router2 = _interopRequireDefault(_Router);

var _ReverseRouter = require('../router/ReverseRouter.js');

var _ReverseRouter2 = _interopRequireDefault(_ReverseRouter);

var _DomainNode = require('../router/DomainNode.js');

var _DomainNode2 = _interopRequireDefault(_DomainNode);

var _MethodNode = require('../router/MethodNode.js');

var _MethodNode2 = _interopRequireDefault(_MethodNode);

var _PathNode = require('../router/PathNode.js');

var _PathNode2 = _interopRequireDefault(_PathNode);

var _NodeFactory = require('../router/NodeFactory.js');

var _NodeFactory2 = _interopRequireDefault(_NodeFactory);

var _ComponentRegister = require('../ComponentRegister.js');

var _ComponentRegister2 = _interopRequireDefault(_ComponentRegister);

var _WebpackCompiler = require('../compiler/webpack/WebpackCompiler.js');

var _WebpackCompiler2 = _interopRequireDefault(_WebpackCompiler);

var _defaultFrameworkConfig = require('../defaultFrameworkConfig.js');

var _Application = require('../Application.js');

var _Application2 = _interopRequireDefault(_Application);

var _Precompiler = require('../precompile/Precompiler.js');

var _Precompiler2 = _interopRequireDefault(_Precompiler);

var _registerRouterNodes = require('soya/lib/server/registerRouterNodes');

var _registerRouterNodes2 = _interopRequireDefault(_registerRouterNodes);

var _createLogger = require('./createLogger.js');

var _createLogger2 = _interopRequireDefault(_createLogger);

var _createErrorHandler = require('./createErrorHandler.js');

var _createErrorHandler2 = _interopRequireDefault(_createErrorHandler);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * @param {Object} config
 * @param {Object} pages
 * @param {Object} components
 * @SERVER
 */
function server(config, pages) {
  var frameworkConfig = config.frameworkConfig;
  var serverConfig = config.serverConfig;
  var clientConfig = config.clientConfig;
  frameworkConfig = _extends({}, _defaultFrameworkConfig.DEFAULT_FRAMEWORK_CONFIG, frameworkConfig);

  var createLogger = _createLogger2.default;
  var createErrorHandler = _createErrorHandler2.default;

  // Load custom logger and error handler factory.
  if (typeof frameworkConfig.loggerFactoryFunction == 'function') {
    createLogger = frameworkConfig.loggerFactoryFunction;
  }
  if (typeof frameworkConfig.errorHandlerFactoryFunction == 'function') {
    createErrorHandler = frameworkConfig.errorHandlerFactorFunction;
  }

  var logger = createLogger(serverConfig);
  var errorHandler = createErrorHandler(serverConfig, logger);
  var nodeFactory = new _NodeFactory2.default();
  var pageName,
      handlerRegister = new _ComponentRegister2.default(logger);

  // Register all pages to handler register.
  for (pageName in pages) {
    handlerRegister.regPage(pageName, _path2.default.join(frameworkConfig.absoluteProjectDir, pageName), pages[pageName]);
  }

  // Register default nodes for router.
  nodeFactory.registerNodeType(_DomainNode2.default);
  nodeFactory.registerNodeType(_MethodNode2.default);
  nodeFactory.registerNodeType(_PathNode2.default);

  // Load custom router nodes and create router.
  (0, _registerRouterNodes2.default)(nodeFactory, serverConfig);
  var router = new _Router2.default(logger, nodeFactory, handlerRegister);
  var reverseRouter = new _ReverseRouter2.default(nodeFactory);

  if (frameworkConfig.componentBrowser) {
    // Register component browser routes.
    router.reg('CMPT_BROWSER_LIST', {
      page: _Precompiler2.default.getCmptBrowserListPageRelativePath(),
      nodes: [['method', 'GET'], ['path', '/component-browser']]
    });
  }

  // Load routes.
  var routes = _jsYaml2.default.safeLoad(_fs2.default.readFileSync(_path2.default.join(config.frameworkConfig.absoluteProjectDir, 'routes.yml'), 'utf8'));
  var routeId;
  for (routeId in routes) {
    router.reg(routeId, routes[routeId]);
    reverseRouter.reg(routeId, routes[routeId]);
  }

  // If not found page is not set up.
  if (!router.getNotFoundRouteResult()) {
    throw new Error('404 not found page not registered! Please create a 404 not found page.');
  }

  var webpackDevMiddleware;
  var webpackHotMiddleware;

  if (frameworkConfig.hotReload) {
    webpackDevMiddleware = require('webpack-dev-middleware');
    webpackHotMiddleware = require('webpack-hot-middleware');
  }

  var compiler = new _WebpackCompiler2.default(logger, frameworkConfig, _webpack2.default, _react2.default, webpackDevMiddleware, webpackHotMiddleware);

  var application = new _Application2.default(logger, handlerRegister, routes, router, reverseRouter, errorHandler, compiler, frameworkConfig, serverConfig, clientConfig);

  // trigger client build without have to start the server as well.
  if (process.env.RUN_MODE && process.env.RUN_MODE === 'buildClient') {
    application.buildClient();
  } else {
    application.start();
  }
}
//# sourceMappingURL=index.js.map